==> reverie_dreams-0.4.3%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/item/weapon/TreasureHuntingRod.java <==
package cc.thonly.reverie_dreams.item.weapon;

import cc.thonly.reverie_dreams.data.ModTags;
import cc.thonly.reverie_dreams.entity.ModEntities;
import cc.thonly.reverie_dreams.entity.misc.OreEspEntity;
import cc.thonly.reverie_dreams.item.base.SwordItem;
import net.fabricmc.fabric.api.tag.convention.v2.ConventionalBlockTags;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1796;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_3222;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_3481;
import net.minecraft.class_3489;
import net.minecraft.class_3730;
import net.minecraft.class_5250;
import net.minecraft.class_6862;
import net.minecraft.class_9886;
import java.util.ArrayList;
import java.util.List;

public class TreasureHuntingRod extends SwordItem {
    public static final List<class_6862<class_2248>> ORE_BLOCK_TAGS = new ArrayList<>();
    public static final class_9886 MATERIAL = new class_9886(ModTags.BlockTypeTag.EMPTY, 300, 4.0f, 4.5f, 5, class_3489.field_52386);

    static {
        ORE_BLOCK_TAGS.add(class_3481.field_23062);
        ORE_BLOCK_TAGS.add(class_3481.field_28988);
        ORE_BLOCK_TAGS.add(class_3481.field_28989);
        ORE_BLOCK_TAGS.add(class_3481.field_28990);
        ORE_BLOCK_TAGS.add(class_3481.field_28991);
        ORE_BLOCK_TAGS.add(class_3481.field_29193);
        ORE_BLOCK_TAGS.add(class_3481.field_29194);
        ORE_BLOCK_TAGS.add(class_3481.field_29195);
        ORE_BLOCK_TAGS.add(ConventionalBlockTags.ORES);
    }

    public TreasureHuntingRod(float attackDamage, float attackSpeed, class_1793 settings) {
        super(MATERIAL, attackDamage, attackSpeed, settings);
    }

    @Override
    public class_1269 method_7836(class_1937 world, class_1657 user, class_1268 hand) {
        if (!world.method_8608()) {
            class_3222 player = (class_3222) user;
            class_1799 stack = player.method_5998(hand);
            class_1796 cooldown = player.method_7357();

            if (player.method_5715()) {
                player.method_6104(hand);

                class_2338 origin = player.method_24515();
                int radius = 8;

                // 记录最近矿物数据
                double minDistance = Double.MAX_VALUE;
                class_2338 closestOrePos = null;
                class_2248 closestOreBlock = null;

                for (class_2338 pos : class_2338.method_10097(
                        origin.method_10069(-radius, -radius, -radius),
                        origin.method_10069(radius, radius, radius))) {

                    if (!world.method_24794(pos)) continue;

                    class_2680 state = world.method_8320(pos);
                    if (isOre(state)) {
                        int dx = pos.method_10263() - origin.method_10263();
                        int dy = pos.method_10264() - origin.method_10264();
                        int dz = pos.method_10260() - origin.method_10260();
                        double distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

                        if (distance < minDistance) {
                            minDistance = distance;
                            closestOrePos = pos.method_10062();
                            closestOreBlock = state.method_26204();
                        }
                    }
                }

                // 找到了矿物
                if (closestOrePos != null) {
                    int dx = closestOrePos.method_10263() - origin.method_10263();
                    int dy = closestOrePos.method_10264() - origin.method_10264();
                    int dz = closestOrePos.method_10260() - origin.method_10260();
                    int roundedDistance = (int) minDistance;

                    class_5250 message = class_2561.method_43469(
                            "message.treasure_hunting_rod.find", roundedDistance, dx, dy, dz
                    ).method_27693(" ").method_10852(class_2561.method_43471(closestOreBlock.method_63499()));

//                    entity.sendMessage(message, false);
                    OreEspEntity oreEspEntity = ModEntities.ORE_ESP_ENTITY_TYPE.method_5883(world, class_3730.field_16467);
                    if (oreEspEntity != null) {
                        oreEspEntity.method_48883(world.method_8320(closestOrePos));
                        oreEspEntity.method_33574(new class_243(closestOrePos));
                        oreEspEntity.method_5834(true);
                        world.method_8649(oreEspEntity);
                    }

                    world.method_43128(null, player.method_23317(), player.method_23320(), player.method_23321(),
                            class_3417.field_14622.comp_349(),
                            class_3419.field_15248, 1.0f, 1.0f);
                } else {
                    player.method_7353(class_2561.method_43471("message.treasure_hunting_rod.not_found"), false);
                }

                // 伤害和冷却
                if (!player.method_56992()) {
                    stack.method_61653(1, player);
                }

                cooldown.method_62835(stack, 35); // 设置冷却
                return class_1269.field_52422;
            }
        }

        return super.method_7836(world, user, hand);
    }


    public static boolean isOre(class_2680 blockState) {
        for (class_6862<class_2248> tag : ORE_BLOCK_TAGS) {
            if (blockState.method_26164(tag)) {
                return true;
            }
        }
        return false;
    }

}

==> reverie_dreams-0.4.6%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/item/weapon/TreasureHuntingRod.java <==
package cc.thonly.reverie_dreams.item.weapon;

import cc.thonly.reverie_dreams.registry.tag.RDBlockTags;
import cc.thonly.reverie_dreams.registry.content.entity.RDEntityTypes;
import cc.thonly.reverie_dreams.entity.misc.OreEspEntity;
import cc.thonly.reverie_dreams.item.base.SwordItem;
import net.fabricmc.fabric.api.tag.convention.v2.ConventionalBlockTags;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1796;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_3222;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_3481;
import net.minecraft.class_3489;
import net.minecraft.class_3730;
import net.minecraft.class_5250;
import net.minecraft.class_6862;
import net.minecraft.class_9886;
import java.util.ArrayList;
import java.util.List;

public class TreasureHuntingRod extends SwordItem {
    public static final List<class_6862<class_2248>> ORE_BLOCK_TAGS = new ArrayList<>();
    public static final class_9886 MATERIAL = new class_9886(RDBlockTags.EMPTY, 300, 4.0f, 4.5f, 5, class_3489.field_52386);

    static {
        ORE_BLOCK_TAGS.add(class_3481.field_23062);
        ORE_BLOCK_TAGS.add(class_3481.field_28988);
        ORE_BLOCK_TAGS.add(class_3481.field_28989);
        ORE_BLOCK_TAGS.add(class_3481.field_28990);
        ORE_BLOCK_TAGS.add(class_3481.field_28991);
        ORE_BLOCK_TAGS.add(class_3481.field_29193);
        ORE_BLOCK_TAGS.add(class_3481.field_29194);
        ORE_BLOCK_TAGS.add(class_3481.field_29195);
        ORE_BLOCK_TAGS.add(ConventionalBlockTags.ORES);
    }

    public TreasureHuntingRod(float attackDamage, float attackSpeed, class_1793 settings) {
        super(MATERIAL, attackDamage, attackSpeed, settings);
    }

    @Override
    public class_1269 method_7836(class_1937 world, class_1657 user, class_1268 hand) {
        if (!world.method_8608()) {
            class_3222 player = (class_3222) user;
            class_1799 stack = player.method_5998(hand);
            class_1796 cooldown = player.method_7357();

            if (player.method_5715()) {
                player.method_6104(hand);

                class_2338 origin = player.method_24515();
                int radius = 8;

                // 记录最近矿物数据
                double minDistance = Double.MAX_VALUE;
                class_2338 closestOrePos = null;
                class_2248 closestOreBlock = null;

                for (class_2338 pos : class_2338.method_10097(
                        origin.method_10069(-radius, -radius, -radius),
                        origin.method_10069(radius, radius, radius))) {

                    if (!world.method_24794(pos)) continue;

                    class_2680 state = world.method_8320(pos);
                    if (isOre(state)) {
                        int dx = pos.method_10263() - origin.method_10263();
                        int dy = pos.method_10264() - origin.method_10264();
                        int dz = pos.method_10260() - origin.method_10260();
                        double distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

                        if (distance < minDistance) {
                            minDistance = distance;
                            closestOrePos = pos.method_10062();
                            closestOreBlock = state.method_26204();
                        }
                    }
                }

                // 找到了矿物
                if (closestOrePos != null) {
                    int dx = closestOrePos.method_10263() - origin.method_10263();
                    int dy = closestOrePos.method_10264() - origin.method_10264();
                    int dz = closestOrePos.method_10260() - origin.method_10260();
                    int roundedDistance = (int) minDistance;

                    class_5250 message = class_2561.method_43469(
                            "message.treasure_hunting_rod.find", roundedDistance, dx, dy, dz
                    ).method_27693(" ").method_10852(class_2561.method_43471(closestOreBlock.method_63499()));

//                    entity.sendMessage(message, false);
                    OreEspEntity oreEspEntity = RDEntityTypes.ORE_ESP_ENTITY_TYPE.method_5883(world, class_3730.field_16467);
                    if (oreEspEntity != null) {
                        oreEspEntity.method_48883(world.method_8320(closestOrePos));
                        oreEspEntity.method_33574(new class_243(closestOrePos));
                        oreEspEntity.method_5834(true);
                        world.method_8649(oreEspEntity);
                    }

                    world.method_43128(null, player.method_23317(), player.method_23320(), player.method_23321(),
                            class_3417.field_14622.comp_349(),
                            class_3419.field_15248, 1.0f, 1.0f);
                } else {
                    player.method_7353(class_2561.method_43471("message.treasure_hunting_rod.not_found"), false);
                }

                // 伤害和冷却
                if (!player.method_56992()) {
                    stack.method_61653(1, player);
                }

                cooldown.method_62835(stack, 35); // 设置冷却
                return class_1269.field_52422;
            }
        }

        return super.method_7836(world, user, hand);
    }


    public static boolean isOre(class_2680 blockState) {
        for (class_6862<class_2248> tag : ORE_BLOCK_TAGS) {
            if (blockState.method_26164(tag)) {
                return true;
            }
        }
        return false;
    }

}
